@@ -31,6 +31,14 @@ static void on_way_add_clicked(GtkButton *button, gpointer data) {
   map_action_set((appdata_t*)data, MAP_ACTION_WAY_ADD);
 }
 
+static void on_way_node_add_clicked(GtkButton *button, gpointer data) {
+  map_action_set((appdata_t*)data, MAP_ACTION_WAY_NODE_ADD);
+}
+
+static void on_way_cut_clicked(GtkButton *button, gpointer data) {
+  map_action_set((appdata_t*)data, MAP_ACTION_WAY_CUT);
+}
+
 static void on_trash_clicked(GtkButton *button, gpointer data) {
   map_delete_selected((appdata_t*)data);
 }
@@ -58,9 +66,15 @@ void icon_bar_map_item_selected(appdata_t *appdata,
   iconbar_t *iconbar = appdata->iconbar;
 
   gtk_widget_set_sensitive(iconbar->info, selected);
-
-  /* currently only nodes can be deleted */
   gtk_widget_set_sensitive(iconbar->trash, selected);
+
+  if(selected && map_item && map_item->type == MAP_TYPE_WAY) {
+    gtk_widget_set_sensitive(iconbar->way_node_add, TRUE);
+    gtk_widget_set_sensitive(iconbar->way_cut, TRUE);
+  } else {
+    gtk_widget_set_sensitive(iconbar->way_node_add, FALSE);
+    gtk_widget_set_sensitive(iconbar->way_cut, FALSE);
+  }
 }
 
 /* if a user action is on progress, then disable all buttons that */
@@ -73,8 +87,21 @@ void icon_bar_map_action_idle(appdata_t *appdata, gboolean idle) {
     NULL
   };
 
-  for(i=0;action_widgets[i];i++) {
+  for(i=0;action_widgets[i];i++) 
     gtk_widget_set_sensitive(action_widgets[i], idle);
+
+  /* special handling for icons that depend on further state */
+  if(!idle) {
+    gtk_widget_set_sensitive(appdata->iconbar->way_node_add, FALSE);
+    gtk_widget_set_sensitive(appdata->iconbar->way_cut, FALSE);
+  } else {
+    if(appdata->map->selected.type == MAP_TYPE_WAY) {
+      gtk_widget_set_sensitive(appdata->iconbar->way_node_add, TRUE);
+      gtk_widget_set_sensitive(appdata->iconbar->way_cut, TRUE);
+    } else {
+      gtk_widget_set_sensitive(appdata->iconbar->way_node_add, FALSE);
+      gtk_widget_set_sensitive(appdata->iconbar->way_cut, FALSE);
+    }
   }
 }
 
@@ -118,14 +145,29 @@ GtkWidget *iconbar_new(appdata_t *appdata) {
   gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 
   /* -------------------------------------------------------- */
-  iconbar->info = tool_add(toolbar, appdata, "info", on_info_clicked);
   iconbar->trash = tool_add(toolbar, appdata, "trash", on_trash_clicked);
 
   /* -------------------------------------------------------- */
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(),-1);
+  iconbar->info = tool_add(toolbar, appdata, "info", on_info_clicked);
+
+  /* -------------------------------------------------------- */
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(),-1);
+
+  iconbar->node_add = tool_add(toolbar, appdata, "node_add", 
+			       on_node_add_clicked);
+
+  /* -------------------------------------------------------- */
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(),-1);
+
+  iconbar->way_add = tool_add(toolbar, appdata, "way_add", 
+			      on_way_add_clicked);
+  iconbar->way_node_add = tool_add(toolbar, appdata, "way_node_add", 
+			      on_way_node_add_clicked);
+  iconbar->way_cut = tool_add(toolbar, appdata, "way_cut", 
+			      on_way_cut_clicked);
+
 
-  iconbar->node_add = tool_add(toolbar, appdata, "node_add", on_node_add_clicked);
-  iconbar->way_add = tool_add(toolbar, appdata, "way_add", on_way_add_clicked);
   gtk_box_pack_start(GTK_BOX(vbox), toolbar, TRUE, TRUE, 0);
 
   /* -------------------------------------------------------- */
