@@ -376,17 +376,26 @@ void info_dialog(appdata_t *appdata) {
   /* ------------ coordinate (only for nodes) ----------------- */
   if(appdata->map->selected.type == MAP_TYPE_NODE) {
     char pos_str[32];
-    pos_lat_str(pos_str, sizeof(pos_str), appdata->map->selected.node->pos.lat);
+    pos_lat_str(pos_str, sizeof(pos_str),appdata->map->selected.node->pos.lat);
     label = gtk_label_new(pos_str);
     gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 1, 2);
-    pos_lat_str(pos_str, sizeof(pos_str), appdata->map->selected.node->pos.lon);
+    pos_lat_str(pos_str, sizeof(pos_str),appdata->map->selected.node->pos.lon);
     label = gtk_label_new(pos_str);
     gtk_table_attach_defaults(GTK_TABLE(table),  label, 1, 2, 1, 2);
-  }
+  } else {
+    char *nodes_str = g_strdup_printf(_("Length: %u nodes"), 
+		osm_way_number_of_nodes(appdata->map->selected.way));
+    label = gtk_label_new(nodes_str);
+    gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 1, 2);
+    g_free(nodes_str);
 
-  /* ------------------- presets button -------------------- */
-  //  gtk_table_attach_defaults(GTK_TABLE(table),
-  //		    josm_presets_select(appdata, context), 0, 2, 2, 3);
+    if(appdata->map->selected.way->draw_flags & OSM_DRAW_FLAGS_POLYGON) 
+      label = gtk_label_new("Closed way (polygon)");
+    else
+      label = gtk_label_new("Open way (line)");
+
+    gtk_table_attach_defaults(GTK_TABLE(table),  label, 1, 2, 1, 2);
+  }
 
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(context->dialog)->vbox), table, 
 		     FALSE, FALSE, 0);
