@@ -548,13 +548,48 @@ cb_menu_item(GtkMenuItem *menu_item, gpointer data) {
   if(tag) {
     tag_context_t *tag_context = context->tag_context;
 
+    /* add new tags to the old list and replace entries with the same key */
+
+    while(tag) {
+#if 0
+      printf("current:\n");
+      tag_t *mdst = &tag_context->tag;
+      while(mdst) {
+	printf("%s: %s\n", msdt
+#endif      
+
+      tag_t *next = tag->next;
+      tag->next = NULL;
+
+      tag_t **dst = tag_context->tag;
+      gboolean replaced = FALSE;
+      while(*dst && !replaced) {
+	if(strcasecmp((*dst)->key, tag->key) == 0) {
+	  g_free((*dst)->value);
+	  (*dst)->value = g_strdup(tag->value);
+	  replaced = TRUE;
+	}
+	dst = &(*dst)->next;
+      }
+
+      /* if nothing was replaced, then just append new tag */
+      if(!replaced) 
+	*dst = tag;
+      else
+	osm_tag_free(tag);
+      
+      tag = next;
+    }
+
+#if 0
     /* free existing tags */
     osm_tags_free(*tag_context->tag);
 
     /* attach new tags */
     *tag_context->tag = tag;
+#endif
 
-    info_tags_replace(tag_context, tag);
+    info_tags_replace(tag_context);
   }
 }
 
