@@ -1340,55 +1340,69 @@ static void map_button_release(map_t *map, gint x, gint y) {
       
     map_item_deselect(map->appdata);
 
-    if(node)
+    if(node) {
       map_node_select(map->appdata, node);
+
+      /* let the user specify some tags for the new node */
+      info_dialog(map->appdata);
+    }
     break;
 
   case MAP_ACTION_WAY_ADD:
     printf("released after WAY ADD\n");
     map_hl_cursor_clear(map);
 
-    /* use the existing node if one was touched */
-    node = map_hl_touchnode_get_node(map);
-    if(node) {
-      printf("  re-using node #%ld\n", node->id);
-      map_hl_touchnode_clear(map);
+    /* convert mouse position to canvas (world) position */
+    canvas_window2world(map->canvas, x, y, &x, &y);
+
+    /* check if this was a double click. This is the case if */
+    /* the last node placed is less than 5 pixels from the current */
+    /* position */
+    node_t *lnode = osm_way_get_last_node(map->action.way);
+    if(lnode && (map->zoom * sqrt((lnode->lpos.x-x)*(lnode->lpos.x-x)+ 
+				  (lnode->lpos.y-y)*(lnode->lpos.y-y))) < 5) {
+      printf("detected double click -> simulate ok click\n");
+      map_action_ok(map->appdata);
     } else {
-
-      /* convert mouse position to canvas (world) position */
-      canvas_window2world(map->canvas, x, y, &x, &y);
-
-      if(!osm_position_within_bounds(map->appdata->osm, x, y)) 
-	map_outside_error(map->appdata);
-      else 
-	node = osm_node_new(map->appdata->osm, x, y);
-    }
-
-    if(node) {
-      g_assert(map->action.way);
-      osm_way_append_node(map->action.way, node);
-
-      switch(osm_node_chain_length(map->action.way->node_chain)) {
-      case 1:
-	/* replace "place first node..." message */
-	statusbar_set(map->appdata, _("Place next node of way"));
-	break;
+      
+      /* use the existing node if one was touched */
+      node = map_hl_touchnode_get_node(map);
+      if(node) {
+	printf("  re-using node #%ld\n", node->id);
+	map_hl_touchnode_clear(map);
+      } else {
+	if(!osm_position_within_bounds(map->appdata->osm, x, y)) 
+	  map_outside_error(map->appdata);
+	else 
+	  node = osm_node_new(map->appdata->osm, x, y);
+      }
+      
+      if(node) {
+	g_assert(map->action.way);
+	osm_way_append_node(map->action.way, node);
 	
-      case 2:
-	/* two nodes are enough for a valid way */
-	icon_bar_map_cancel_ok(map->appdata, TRUE, TRUE);
-	break;
+	switch(osm_node_chain_length(map->action.way->node_chain)) {
+	case 1:
+	  /* replace "place first node..." message */
+	  statusbar_set(map->appdata, _("Place next node of way"));
+	  break;
+	  
+	case 2:
+	  /* two nodes are enough for a valid way */
+	  icon_bar_map_cancel_ok(map->appdata, TRUE, TRUE);
+	  break;
+	  
+	default:
+	  break;
+	}
 	
-      default:
-	break;
+	/* remove prior version of this way */
+	map_item_chain_destroy(&map->action.way->map_item_chain);
+	
+	/* draw current way */
+	map_way_color(map->appdata->potlatch, map->action.way);
+	map_way_draw(map, map->action.way);
       }
-
-      /* remove prior version of this way */
-      map_item_chain_destroy(&map->action.way->map_item_chain);
-
-      /* draw current way */
-      map_way_color(map->appdata->potlatch, map->action.way);
-      map_way_draw(map, map->action.way);
     }
     break;
 
@@ -1458,11 +1472,20 @@ static void map_button_release(map_t *map, gint x, gint y) {
       canvas_window2world(map->canvas, x, y, &x, &y);
       gint insert_after = canvas_item_get_segment(item->item, x, y);
       if(insert_after >= 0) {
-
 	/* create a duplicate of the currently selected way */
 	way_t *way = item->way;
 	way_t *new = osm_way_new();
-    
+
+	/* if this is a closed way, reorder (rotate) it, so the */
+	/* place to cut is adjecent to the begin/end of the way. */
+	/* this prevents a cut polygon to be split into two ways */
+	g_assert(way->node_chain);
+	if(way->node_chain->node == osm_way_get_last_node(way)) {
+	  printf("CLOSED WAY -> rotate by %d\n", insert_after);
+	  osm_way_rotate(way, insert_after);
+	  insert_after = 0;
+	}
+
 	/* ------------  copy all tags ------------- */
 	tag_t **dst_tag = &new->tag;
 	tag_t *src_tag = way->tag;
@@ -1529,7 +1552,16 @@ static void map_button_release(map_t *map, gint x, gint y) {
 	printf("remove visible version of way #%ld\n", way->id);
 	map_item_chain_destroy(&way->map_item_chain);
 
-	/* one of the way may only consist of a single node. */
+	/* swap chains of the old way is to be destroyed due to a lack */
+	/* of nodes */
+	if(osm_way_number_of_nodes(way) < 2) {
+	  printf("swapping ways to avoid destruction of original way\n");
+	  node_chain_t *tmp = way->node_chain;
+	  way->node_chain = new->node_chain;
+	  new->node_chain = tmp;
+	}
+
+	/* the way may still only consist of a single node. */
 	/* remove it then */
 	if(osm_way_number_of_nodes(way) < 2) {
 	  printf("original way has less than 2 nodes left, deleting it\n");
@@ -1700,15 +1732,19 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
       map_do_scroll_step(appdata->map, 0, +50);
       break;
 
-#ifdef USE_HILDON
-    case HILDON_HARDKEY_SELECT:
-#else
-    case 'i':
-#endif
-      if(GTK_WIDGET_FLAGS(appdata->iconbar->info) & GTK_SENSITIVE)
+    case GDK_Return:   // same as HILDON_HARDKEY_SELECT
+      /* if the ok button is enabled, call its function */
+      if(GTK_WIDGET_FLAGS(appdata->iconbar->ok) & GTK_SENSITIVE) 
+	map_action_ok(appdata);
+      /* otherwise if info is enabled call that */
+      else if(GTK_WIDGET_FLAGS(appdata->iconbar->info) & GTK_SENSITIVE)
 	info_dialog(appdata);
-      else
-	printf("info button not enabled\n");
+      break;
+
+    case GDK_Escape:   // same as HILDON_HARDKEY_ESC
+      /* if the cancel button is enabled, call its function */
+      if(GTK_WIDGET_FLAGS(appdata->iconbar->cancel) & GTK_SENSITIVE) 
+	map_action_cancel(appdata);
       break;
 
 #ifdef USE_HILDON
@@ -2014,6 +2050,10 @@ void map_action_ok(appdata_t *appdata) {
     map_way_select(appdata, map->action.way);
 
     map->action.way = NULL;
+
+    /* let the user specify some tags for the new way */
+    info_dialog(appdata);
+
     break;
 
   case MAP_ACTION_BG_ADJUST:
