@@ -67,6 +67,8 @@ typedef enum {
   MAP_ACTION_NODE_ADD, 
   MAP_ACTION_BG_ADJUST,
   MAP_ACTION_WAY_ADD,
+  MAP_ACTION_WAY_NODE_ADD,
+  MAP_ACTION_WAY_CUT,
   MAP_ACTION_NUM
 } map_action_t;
 
