@@ -23,8 +23,9 @@
 void map_hl_cursor_draw(map_t *map, gint x, gint y);
 void map_hl_cursor_clear(map_t *map);
 
-void map_hl_touchnode_draw(map_t *map, gint x, gint y);
+void map_hl_touchnode_draw(map_t *map, node_t *node);
 void map_hl_touchnode_clear(map_t *map);
+node_t *map_hl_touchnode_get_node(map_t *map);
 
 void map_hl_remove(appdata_t *appdata);
 gboolean map_hl_item_is_highlighted(map_t *map, map_item_t *item);
