@@ -1618,3 +1618,36 @@ void osm_way_revert(way_t *way) {
 
   way->node_chain = new;
 }
+
+node_t *osm_way_get_last_node(way_t *way) {
+  node_chain_t *chain = way->node_chain;
+
+  while(chain && chain->next) chain=chain->next;
+
+  if(!chain) return NULL;
+
+  return chain->node;
+}
+
+void osm_way_rotate(way_t *way, gint offset) {
+  if(!offset) return;
+
+  /* needs at least two nodes to work properly */
+  g_assert(way->node_chain);
+  g_assert(way->node_chain->next);
+
+  while(offset--) {
+    node_chain_t *chain = way->node_chain;
+    chain->node->ways--; // reduce way count of old start/end node
+
+    /* move all nodes ahead one chain element ... */
+    while(chain->next) {
+      chain->node = chain->next->node;
+      chain = chain->next;
+    }
+
+    /* ... and make last one same as first one */
+    chain->node = way->node_chain->node;
+    chain->node->ways++; // increase way count of new start/end node
+  }
+}
