@@ -518,24 +518,8 @@ static void osm_upload_nodes(osm_upload_context_t *context) {
     /* make sure gui gets updated */
     while(gtk_events_pending()) gtk_main_iteration();
 
-    if(node->flags & OSM_FLAG_DELETED) {
-      printf("deleting node on server\n");
-
-      appendf(&context->log, _("Delete node #%ld "), node->id);
-
-      char *url = g_strdup_printf("%s/node/%lu", project->server, node->id);
-      char *cred = g_strdup_printf("%s:%s", project->username, 
-				   project->password);
-
-      if(osm_delete_item(&context->log, url, cred)) {
-	node->flags &= ~(OSM_FLAG_DIRTY | OSM_FLAG_DELETED);
-	project->data_dirty = TRUE;
-      }
-      
-      g_free(cred);
-    }
-    
-    if(node->flags & (OSM_FLAG_DIRTY | OSM_FLAG_NEW)) {
+    if((node->flags & (OSM_FLAG_DIRTY | OSM_FLAG_NEW)) &&
+       (!(node->flags & OSM_FLAG_DELETED))) {
       char *url = NULL;
 
       if(node->flags & OSM_FLAG_NEW) {
@@ -604,20 +588,8 @@ static void osm_upload_ways(osm_upload_context_t *context) {
     /* make sure gui gets updated */
     while(gtk_events_pending()) gtk_main_iteration();
 
-    /* check if way only consists of "clean" nodes */
-    gboolean clean = TRUE;
-    node_chain_t *chain = way->node_chain;
-    while(chain) {
-      if(chain->node->flags & (OSM_FLAG_DIRTY | OSM_FLAG_NEW))
-	clean = FALSE;
-      
-      chain = chain->next;
-    }
-
-    if(!clean) 
-      appendf(&context->log, _("skip (contains dirty nodes)\n"));
-
-    if(clean && way->flags & (OSM_FLAG_DIRTY | OSM_FLAG_NEW)) {
+    if((way->flags & (OSM_FLAG_DIRTY | OSM_FLAG_NEW)) &&
+       (!(way->flags & OSM_FLAG_DELETED))) {
       char *url = NULL;
       
       if(way->flags & OSM_FLAG_NEW) {
@@ -649,6 +621,47 @@ static void osm_upload_ways(osm_upload_context_t *context) {
 }
 
 
+static void osm_upload_relations(osm_upload_context_t *context) {
+  relation_t *relation = context->osm->relation;
+  project_t *project = context->project;
+
+  while(relation) {
+    /* make sure gui gets updated */
+    while(gtk_events_pending()) gtk_main_iteration();
+
+    if((relation->flags & (OSM_FLAG_DIRTY | OSM_FLAG_NEW)) &&
+       (!(relation->flags & OSM_FLAG_DELETED))) {
+      char *url = NULL;
+      
+      if(relation->flags & OSM_FLAG_NEW) {
+	url = g_strdup_printf("%s/relation/create", project->server);
+	appendf(&context->log, _("New relation "));
+      } else {
+	url = g_strdup_printf("%s/relation/%lu", project->server,relation->id);
+	appendf(&context->log, _("Modified relation #%ld "), relation->id);
+      }
+      
+      /* upload this relation */
+      char *xml_str = osm_generate_xml_relation(context->osm, relation);
+      if(xml_str) {
+	printf("uploading relation %s from address %p\n", url, xml_str);
+	
+	char *cred = g_strdup_printf("%s:%s", 
+				     project->username, project->password);
+	if(osm_update_item(&context->log, xml_str, url, cred, 
+		   (relation->flags & OSM_FLAG_NEW)?&(relation->id):NULL)) {
+	  relation->flags &= ~(OSM_FLAG_DIRTY | OSM_FLAG_NEW);
+	  project->data_dirty = TRUE;
+	}
+	g_free(cred);
+      }
+      g_free(url);
+    }
+    relation = relation->next;
+  }
+}
+
+
 void osm_upload(appdata_t *appdata, osm_t *osm, project_t *project) {
 
   printf("starting upload\n");
@@ -668,7 +681,7 @@ void osm_upload(appdata_t *appdata, osm_t *osm, project_t *project) {
 
     node = node->next;
   }
-  printf("nodes: new %d, dirty %d, deleted %d\n",
+  printf("nodes:     new %2d, dirty %2d, deleted %2d\n",
 	 dirty.nodes.new, dirty.nodes.dirty, dirty.nodes.deleted);
   
   /* count ways */
@@ -681,7 +694,7 @@ void osm_upload(appdata_t *appdata, osm_t *osm, project_t *project) {
 
     way = way->next;
   }
-  printf("ways: new %d, dirty %d, deleted %d\n",
+  printf("ways:      new %2d, dirty %2d, deleted %2d\n",
 	 dirty.ways.new, dirty.ways.dirty, dirty.ways.deleted);
 
   /* count relations */
@@ -694,7 +707,7 @@ void osm_upload(appdata_t *appdata, osm_t *osm, project_t *project) {
 
     relation = relation->next;
   }
-  printf("relations: new %d, dirty %d, deleted %d\n",
+  printf("relations: new %2d, dirty %2d, deleted %2d\n",
 	 dirty.relations.new, dirty.relations.dirty, dirty.relations.deleted);
 
 
@@ -818,11 +831,13 @@ void osm_upload(appdata_t *appdata, osm_t *osm, project_t *project) {
   osm_upload_nodes(context);
   appendf(&context->log, _("Uploading ways:\n"));
   osm_upload_ways(context);
+  appendf(&context->log, _("Uploading relations:\n"));
+  osm_upload_relations(context);
   appendf(&context->log, _("Deleting ways:\n"));
   osm_delete_ways(context);
   appendf(&context->log, _("Deleting nodes:\n"));
   osm_delete_nodes(context);
-  //  appendf(&context->log, "Relations are not supported yet!\n");
+
 
   appendf(&context->log, _("Upload done.\n"));
 
