@@ -561,6 +561,7 @@ static gboolean wms_server_dialog(appdata_t *appdata, wms_t *wms) {
   gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
   gtk_table_attach(GTK_TABLE(table),  label, 0, 1, 0, 1, 0,0,0,0);
   GtkWidget *server_entry = gtk_entry_new();
+  HILDON_ENTRY_NO_AUTOCAP(server_entry);
   gtk_entry_set_text(GTK_ENTRY(server_entry), wms->server);
   gtk_table_attach_defaults(GTK_TABLE(table), server_entry, 1, 2, 0, 1);
 
@@ -568,6 +569,7 @@ static gboolean wms_server_dialog(appdata_t *appdata, wms_t *wms) {
   gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
   gtk_table_attach(GTK_TABLE(table),  label, 0, 1, 1, 2, 0,0,0,0);
   GtkWidget *path_entry = gtk_entry_new();
+  HILDON_ENTRY_NO_AUTOCAP(path_entry);
   gtk_entry_set_text(GTK_ENTRY(path_entry), wms->path);
   gtk_table_attach_defaults(GTK_TABLE(table), path_entry, 1, 2, 1, 2);
 
@@ -763,7 +765,7 @@ void wms_import(appdata_t *appdata) {
   }
 
   /* this cancels any wms adjustment in progress */
-  if(appdata->map->action == MAP_ACTION_BG_ADJUST)
+  if(appdata->map->action.type == MAP_ACTION_BG_ADJUST)
     map_action_cancel(appdata);
 
   wms_t *wms = g_new0(wms_t,1);
@@ -1010,7 +1012,7 @@ void wms_remove(appdata_t *appdata) {
   int i=0;
 
   /* this cancels any wms adjustment in progress */
-  if(appdata->map->action == MAP_ACTION_BG_ADJUST)
+  if(appdata->map->action.type == MAP_ACTION_BG_ADJUST)
     map_action_cancel(appdata);
 
   gtk_widget_set_sensitive(appdata->menu_item_wms_clear, FALSE);
